package com.tcctlo.law.mapper;

import com.tcctlo.law.entity.CollectionRecord;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.tcctlo.law.vo.CollectionRecordEnterVO;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 收款记录表 Mapper 接口
 * </p>
 *
 * @author 张世琪
 * @since 2025-02-06
 */
public interface CollectionRecordMapper extends BaseMapper<CollectionRecord> {

    /**
     * 根据案件ID查询收款记录
     * @param caseId 案件ID
     * @return 结果集
     */
    List<CollectionRecordEnterVO> selectByCaseId(@Param("caseId") Long caseId);

    /**
     * 根据案件ID批量删除收款记录
     * @param caseIds 案件ID集合
     * @return 删除结果
     */
    int removeByCaseId(@Param("caseIds") List<Long> caseIds);

    /**
     * 根据缴费记录ID删除缴费记录
     * @param id 缴费记录ID
     * @return 删除结果
     */
    Boolean deleteLogic(@Param("id") Long id);

    /**
     * 根据案件ID和收款记录ID删除收款记录
     * @param id 收款记录ID
     * @param caseId 案件ID
     * @return 结果集
     */
    int removeByCaseIdAdnRecordId(@Param("id") Long id, @Param("caseId") Long caseId);


    /**
     * 收益记录趋势
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @return 结果集
     */
    List<CollectionRecord> earningsTrend(@Param("userId") String userId, @Param("startTime") Date startTime, @Param("endTime") Date endTime);

    /**
     * 收费记录审核
     * @param id id
     * @return 结果集
     */
    int audit(@Param("id") Long id);
}
