package com.tcctlo.law.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctlo.common.core.domain.entity.SysRole;
import com.tcctlo.common.core.domain.entity.SysUser;
import com.tcctlo.common.core.domain.model.LoginUser;
import com.tcctlo.common.utils.SecurityUtils;
import com.tcctlo.common.utils.StringUtils;
import com.tcctlo.law.CaseEnum;
import com.tcctlo.law.entity.*;
import com.tcctlo.law.mapper.*;
import com.tcctlo.law.searcher.CaseCloseSearcher;
import com.tcctlo.law.searcher.CaseInformationEnterSearcher;
import com.tcctlo.law.searcher.CollectionRecordSearcher;
import com.tcctlo.law.service.*;
import com.tcctlo.law.tools.RedisGenerateCaseNo;
import com.tcctlo.law.tools.RichTextToWordSimple;
import com.tcctlo.law.vo.*;
import com.tcctlo.system.mapper.SysUserMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;

/**
 * <p>
 * 案件信息表 服务实现类
 * </p>
 *
 * @author 张世琪
 * @since 2025-02-06
 */
@Service
public class CaseInformationServiceImpl extends ServiceImpl<CaseInformationMapper, CaseInformation> implements ICaseInformationService {

    @Resource
    private RichTextToWordSimple richTextToWordSimple;

    @Resource
    private CaseInformationMapper caseInformationMapper;

    @Resource
    private ICaseInformationService caseInformationService;

    @Resource
    private ImpulseInformationMapper impulseInformationMapper;

    @Resource
    private DemandPaymentRecordMapper demandPaymentRecordMapper;

    @Autowired
    private RedisGenerateCaseNo redisGenerateCaseNo;

    /**
     * 系统用户mapper
     */
    @Resource
    private SysUserMapper sysUserMapper;

    /**
     * 利冲信息mapper
     */
    @Resource
    private ImpulseInformationMapper informationMapper;

    /**
     * 利冲信息service
     */
    @Resource
    private IImpulseInformationService impulseInformationService;

    /**
     * 案件关联律师mapper
     */
    @Resource
    private CaseLawyerMapper caseLawyerMapper;

    /**
     * 案件关联律师service
     */
    @Resource
    private ICaseLawyerService caseLawyerService;

    /**
     * 文件管理mapper
     */
    @Resource
    private FileManagerMapper fileManagerMapper;

    /**
     * 文件管理service
     */
    @Resource
    private IFileManagerService fileManagerService;

    /**
     * 收款记录mapper
     */
    @Resource
    private CollectionRecordMapper collectionRecordMapper;

    /**
     * 收款记录service
     */
    @Resource
    private ICollectionRecordService collectionRecordService;

    /**
     * 案件开票
     */
    @Resource
    private InvoiceRecordMapper invoiceRecordMapper;


    /*@Override
    public Page<CaseInformation> list(Integer pageNo, Integer pageSize, Map<String, Object> params) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色是【代理律师】或【协办律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        if ("ATTORNEY_AGENT".equals(roleKey) || "ASSISTING_LAWYER".equals(roleKey)){
            params.put("userId", loginUser.getUserId().toString());
        }
        Page<CaseInformation> page = new Page<>(pageNo, pageSize);
        return caseInformationMapper.selectCondition(page, params);
    }*/


    /*@Override
    public Page<CaseInformationEnterVO> list(Integer pageNo, Integer pageSize, Map<String, Object> params) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色是【代理律师】或【协办律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        if ("ATTORNEY_AGENT".equals(roleKey) || "ASSISTING_LAWYER".equals(roleKey)){
            params.put("userId", loginUser.getUserId().toString());
        }
        Page<CaseInformation> page = new Page<>(pageNo, pageSize);
        List<CaseInformation> entityLists = caseInformationMapper.selectCondition(page, params).getRecords();
        Page<CaseInformationEnterVO> resPage = new Page<>();
        List<CaseInformationEnterVO> caseInfoVO = new ArrayList<>();
        for (CaseInformation item : entityLists) {
            CaseInformationEnterVO itemVO = new CaseInformationEnterVO();
            itemVO.setCaseInformation(item);
            itemVO.setEntrustingParty(informationMapper.selectByCaseId(item.getId(), 0));    //利冲信息-委托方信息--list
            itemVO.setOpposite(informationMapper.selectByCaseId(item.getId(), 1));    //利冲信息-相对方--list
            itemVO.setAttorneyAgent(caseLawyerMapper.selectAttorneyAgentLawByCaseId(item.getId()));    //代理律师--entity
            itemVO.setAssistingLawyer(caseLawyerMapper.selectAssistingLawyerByCaseId(item.getId()));    //协办律师--list
            caseInfoVO.add(itemVO);
        }
        resPage.setRecords(caseInfoVO);
        return resPage;
    }*/

    @Override
    public Page<CaseInformationListVO> list(Integer pageNo, Integer pageSize, Map<String, Object> params) {
        if (params == null) {
            params = new HashMap<>();
        }
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色【律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        if (CaseEnum.LAWYER.getRoleKey().equals(roleKey)) {
            params.put("userId", loginUser.getUserId().toString());
        }
        Page<CaseInformationListVO> page = new Page<>(pageNo, pageSize);
        Page<CaseInformationListVO> caseInformationPage = caseInformationMapper.selectCondition(page, params);
        for (CaseInformationListVO item : caseInformationPage.getRecords()) {

            Date paymentDeadline = item.getPaymentDeadline();
            if (paymentDeadline != null) {
                if (DateUtil.compare(paymentDeadline, new Date()) < 0 && item.getAmountReceivedSum().compareTo(item.getAmountReceivable()) < 0) {
                    caseInformationMapper.updateCollectionStatus(item.getId(), 3);
                }
            }

            List<ImpulseInformation> entrustingParty = informationMapper.selectByCaseId(item.getId(), 0);
            if (CollUtil.isNotEmpty(entrustingParty)) {
                item.setEntrustingParty(entrustingParty.stream().map(ImpulseInformation::getCaseName).collect(Collectors.joining(",")));
            }

            List<ImpulseInformation> opposite = informationMapper.selectByCaseId(item.getId(), 1);
            if (CollUtil.isNotEmpty(opposite)) {
                item.setOpposite(opposite.stream().map(ImpulseInformation::getCaseName).collect(Collectors.joining(",")));
            }

//            CaseLawyer attorneyAgent = caseLawyerMapper.selectAttorneyAgentLawByCaseId(item.getId());
//            if (BeanUtil.isNotEmpty(attorneyAgent)) {
//                item.setAttorneyAgent(attorneyAgent.getLawyerName());
//            }
            //处理委托律师及协办律师姓名
            List<SysUser> sysUserList = sysUserMapper.queryUserList();
            Map<Long, String> userIdAndNameMap = new HashMap<>();
            if (!CollectionUtils.isEmpty(sysUserList)) {
                for (SysUser sysUser : sysUserList) {
                    userIdAndNameMap.put(sysUser.getUserId(), sysUser.getNickName());
                }
            }
            item.setEntrustedLawyerName(userIdAndNameMap.get(item.getEntrustedLawyerId()));
            // 考虑协办律师可能为空的情况，需要进行判空处理
            if (!StringUtils.isEmpty(item.getCoLawyerIds())) {
                List<String> coLawyerIds = Arrays.asList(item.getCoLawyerIds().split(","));
                List<String> coLawyerNames = new ArrayList<>();
                for (String coLawyerId : coLawyerIds) {
                    coLawyerNames.add(userIdAndNameMap.get(Long.parseLong(coLawyerId)));
                }
                item.setCoLawyerNames(StringUtils.join(coLawyerNames, ","));
            }

//            List<CaseLawyer> assistingLawyer = caseLawyerMapper.selectAssistingLawyerByCaseId(item.getId());
//            if (CollUtil.isNotEmpty(assistingLawyer)) {
//                item.setAssistingLawyer(assistingLawyer.stream().map(CaseLawyer::getLawyerName).collect(Collectors.joining(",")));
//            }

            item.setUncollectedMoney(item.getAmountReceivable().subtract(item.getAmountReceivedSum()));

            int fileNum = caseInformationMapper.selectCaseFileNum(item.getId());
            item.setFileFlag(fileNum > 0);
            item.setFileList(fileManagerMapper.selectFileByCaseId(item.getId()));

            //开票状态
            List<InvoiceRecordVO> invoiceRecords = invoiceRecordMapper.selectByCaseId(item.getId());
            if (invoiceRecords.size() > 0) {
                String invoiceMonet = invoiceRecords.stream().map(InvoiceRecord::getInvoiceMoney).collect(Collectors.toList()).stream().reduce(BigDecimal.ZERO, BigDecimal::add).toString();
                item.setInvoiceStatus(invoiceMonet);
            } else {
                item.setInvoiceStatus("0");
            }

            item.setDemandPaymentRecordList(demandPaymentRecordMapper.selectByCaseId(item.getId()));
        }
        return caseInformationPage;
    }


    @Override
    public CaseInformationListVO selectCaseById(Long id) {
        CaseInformationListVO item = caseInformationMapper.selectCaseById(id);
        List<SysUser> sysUserList = sysUserMapper.queryUserList();
        Map<Long, String> userIdAndNameMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(sysUserList)) {
            for (SysUser sysUser : sysUserList) {
                userIdAndNameMap.put(sysUser.getUserId(), sysUser.getNickName());
            }
        }
        item.setEntrustedLawyerName(userIdAndNameMap.get(item.getEntrustedLawyerId()));
        // 考虑协办律师可能为空的情况，需要进行判空处理
        if (!StringUtils.isEmpty(item.getCoLawyerIds())) {
            List<String> coLawyerIds = Arrays.asList(item.getCoLawyerIds().split(","));
            List<String> coLawyerNames = new ArrayList<>();
            for (String coLawyerId : coLawyerIds) {
                coLawyerNames.add(userIdAndNameMap.get(Long.parseLong(coLawyerId)));
            }
            item.setCoLawyerNames(StringUtils.join(coLawyerNames, ","));
        }
        return item;
    }

    @Override
    public CaseInformation getById(Long id) {
        return caseInformationMapper.selectById(id);
    }


    @Override
    public Integer create(CaseInformation caseInformation) {
        return caseInformationMapper.insert(caseInformation);
    }

    @Override
    public Integer delete(Long id) {
        return caseInformationMapper.deleteById(id);
    }

    @Override
    public Integer update(CaseInformation caseInformation) {
        return caseInformationMapper.updateById(caseInformation);
    }

    /*@Override
    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    public Boolean enterCaseInfo(CaseInformationEnterVO caseInfo) {
        try {
            CaseInformation caseInformation = caseInfo.getCaseInformation();   //案件信息
            caseInformation.setCaseNo(new GenerateCaseNo().generateCaseNo()); //生成案件编号
            caseInformationMapper.insert(caseInfo.getCaseInformation());    //插入案件信息
            Long caseId = caseInformation.getId();  //获取案件ID

            //插入案件关联利冲信息【设置委托方和相对方】
            List<ImpulseInformation> impulseInformationList = new ArrayList<>();
            impulseInformationList.addAll(caseInfo.getEntrustingParty().stream().peek(item -> {
                item.setCaseId(caseId);
                item.setIsClient(0);
            }).collect(Collectors.toList()));
            impulseInformationList.addAll(caseInfo.getOpposite().stream().peek(item -> {
                item.setCaseId(caseId);
                item.setIsClient(1);
            }).collect(Collectors.toList()));
            informationService.saveBatch(impulseInformationList);
            List<Long> impulseInformationIds = impulseInformationList.stream().map(item -> item.getId()).collect(Collectors.toList());
            caseInformation.setImpulseInformationIds(StringUtils.join(impulseInformationIds, ","));

            //插入案件关联律师数据
            List<CaseLawyer> caseLawyerList = new ArrayList<>();
            CaseLawyer attorneyAgent = caseInfo.getAttorneyAgent();
            attorneyAgent.setLawyerType(0);
            attorneyAgent.setCaseId(caseId);
            caseLawyerList.add(attorneyAgent); //代理律师

            caseLawyerList.addAll(caseInfo.getAssistingLawyer().stream().peek(item -> {
                item.setCaseId(caseId);
                item.setLawyerType(1);
            }).collect(Collectors.toList()));   //协办律师
            caseLawyerService.saveBatch(caseLawyerList);
            List<Long> caseLawyerIds = caseLawyerList.stream().map(item -> item.getLawyerId()).collect(Collectors.toList());
            caseInformation.setCaseLawyerIds(StringUtils.join(caseLawyerIds, ","));
            caseInformationMapper.updateById(caseInformation); //更新案件表中的律师IDS和利冲信息IDS

            //插入案件关联收款记录信息【收款记录文件插入】
            List<CollectionRecordEnterVO> collectionRecordList = caseInfo.getCollectionRecordList();
            collectionRecordList.forEach(item -> {
                item.setCaseId(caseId);
                List<FileManager> fileManagerList = item.getFileManager().stream().peek(i -> {
                    i.setCaseId(caseId);
                    i.setFileType(1);   //设置文件类型为【收款记录文件】
                }).collect(Collectors.toList());
                fileManagerService.saveBatch(fileManagerList);
                List<Long> fileIds = fileManagerList.stream().map(FileManager::getId).collect(Collectors.toList());
                item.setCollectionUrlId(StringUtils.join(fileIds, ","));
                collectionRecordMapper.insert(item);
            });
        } catch (Exception e) {
            throw new RuntimeException("案件录入失败");
        }
        return true;
    }*/

    public String getCaseClashInfo(List<ImpulseInformation> list) {
        List<ImpulseInformation> resList = new ArrayList<>();
        ArrayList<ClashInfoVO> clashInfoVOS = new ArrayList<>();
        for (ImpulseInformation item : list) {
            item.setIsClient(1 - item.getIsClient());
            resList.addAll(impulseInformationMapper.retrievalConflict(item));
        }
        for (ImpulseInformation item : resList) {
            //案件名称、委托人\相对方名称、类型【个人/企业】
            ClashInfoVO clashInfoVO = new ClashInfoVO();

            clashInfoVO.setCaseInfoName(item.getCaseName());
        }
        return "null";
    }

//    @Override
//    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
//    public Boolean enterCaseInfo(CaseInformationEnterVO caseInfo) {
//
//        LoginUser loginUser = SecurityUtils.getLoginUser();
//        Long userId = loginUser.getUserId();
//        String username = loginUser.getUsername();
//        String nickName = loginUser.getUser().getNickName();
//        try {
//            CaseInformation caseInformation = caseInfo.getCaseInformation();   //案件信息
//            caseInformation.setCaseNo(redisGenerateCaseNo.generateCaseNo(Integer.parseInt(caseInformation.getBusinessClassify()))); //生成案件编号
//
//
//            caseInformationMapper.insert(caseInfo.getCaseInformation());    //插入案件信息
//            Long caseId = caseInformation.getId();  //获取案件ID
//
//            Date paymentDeadline = caseInformation.getPaymentDeadline();
//            if (paymentDeadline != null) {
//                if (DateUtil.compare(paymentDeadline, new Date()) < 0 && caseInformation.getAmountReceivedSum().compareTo(caseInformation.getAmountReceivable()) < 0) {
//                    caseInformationMapper.updateCollectionStatus(caseId, 3);
//                }
//            }
//
//
//            //插入案件关联利冲信息【设置委托方和相对方】
//            List<ImpulseInformation> impulseInformationList = new ArrayList<>();
//            List<ImpulseInformation> entrustingParty = caseInfo.getEntrustingParty();
//            if (CollUtil.isNotEmpty(entrustingParty)) {
//                impulseInformationList.addAll(caseInfo.getEntrustingParty().stream().peek(item -> {
//                    item.setCaseId(caseId);
//                    item.setIsClient(0);
//                }).collect(Collectors.toList()));
//            }
//            List<ImpulseInformation> opposite = caseInfo.getOpposite();
//            if (CollUtil.isNotEmpty(opposite)) {
//                impulseInformationList.addAll(caseInfo.getOpposite().stream().peek(item -> {
//                    item.setCaseId(caseId);
//                    item.setIsClient(1);
//                }).collect(Collectors.toList()));
//            }
//            impulseInformationService.saveBatch(impulseInformationList);
//            List<Long> impulseInformationIds = impulseInformationList.stream().map(item -> item.getId()).collect(Collectors.toList());
//            caseInformation.setImpulseInformationIds(StringUtils.join(impulseInformationIds, ","));
//
//            //插入案件关联律师数据
//            List<CaseLawyer> caseLawyerList = new ArrayList<>();
//            CaseLawyer attorneyAgent = caseInfo.getAttorneyAgent();
//            if (BeanUtil.isNotEmpty(attorneyAgent)) {
//                attorneyAgent.setLawyerType(0);
//                attorneyAgent.setCaseId(caseId);
//                caseLawyerList.add(attorneyAgent); //代理律师
//            }
//            List<CaseLawyer> assistingLawyer = caseInfo.getAssistingLawyer();
//            if (CollUtil.isNotEmpty(assistingLawyer)) {
//                caseLawyerList.addAll(assistingLawyer.stream().peek(item -> {
//                    item.setCaseId(caseId);
//                    item.setLawyerType(1);
//                }).collect(Collectors.toList()));   //协办律师
//            }
//            caseLawyerService.saveBatch(caseLawyerList);
//            List<Long> caseLawyerIds = caseLawyerList.stream().map(item -> item.getLawyerId()).collect(Collectors.toList());
//            caseInformation.setCaseLawyerIds(StringUtils.join(caseLawyerIds, ","));
//            caseInformationMapper.updateById(caseInformation); //更新案件表中的律师IDS和利冲信息IDS
//
//            //插入案件关联收款记录信息【收款记录文件插入】
//            List<CollectionRecordEnterVO> collectionRecordList = caseInfo.getCollectionRecordList();
//            if (CollUtil.isNotEmpty(collectionRecordList)) {
//                collectionRecordList.forEach(item -> {
//                    item.setCaseId(caseId);
//                    List<FileManager> fileManagerList = item.getFileManager().stream().peek(i -> {
//                        i.setCaseId(caseId);
//                        i.setFileType(1);   //设置文件类型为【收款记录文件】
//                    }).collect(Collectors.toList());
//                    fileManagerService.saveBatch(fileManagerList);
//                    List<Long> fileIds = fileManagerList.stream().map(FileManager::getId).collect(Collectors.toList());
//                    item.setCollectionUrlId(StringUtils.join(fileIds, ","));
//                    item.setUserId(userId);
//                    item.setUserName(nickName);
//                    String collect = loginUser.getUser().getRoles().stream().map(SysRole::getRoleKey).collect(Collectors.joining(","));
//                    if (!collect.contains("FINANCE")) {
//                        item.setAuditStatus(0);
//                    } else {
//                        item.setAuditStatus(1);
//                    }
//                    collectionRecordMapper.insert(item);
//                });
//            }
//        } catch (Exception e) {
//            throw new RuntimeException("案件录入失败");
//        }
//        return true;
//    }

//    @Override
//    public CaseInformationEnterVO selectCaseInfoById(Long id) {
//        CaseInformationEnterVO result = new CaseInformationEnterVO();
//        //案件信息表----caseInformation
//        CaseInformation caseInformation = caseInformationMapper.selectCaseInfoById(id);
//        result.setCaseInformation(caseInformation);
//
//        //设置案件未收金额
//        result.setUncollectedMoney(caseInformation.getAmountReceivable().subtract(caseInformation.getAmountReceivedSum()));
//
//        //案件关联【委托方】----entrustingParty
//        result.setEntrustingParty(informationMapper.selectByCaseId(id, 0));
//
//        //案件关联【相对方】----opposite
//        result.setOpposite(informationMapper.selectByCaseId(id, 1));
//
//        //代理律师----attorneyAgent
//        result.setAttorneyAgent(caseLawyerMapper.selectAttorneyAgentLawByCaseId(id));
//
//        //协办律师----assistingLawyer
//        result.setAssistingLawyer(caseLawyerMapper.selectAssistingLawyerByCaseId(id));
//
//        /**
//         * 收款记录----collectionRecordList
//         * 收款记录基本信息
//         * 收款记录文件
//         */
//        List<CollectionRecordEnterVO> records = collectionRecordMapper.selectByCaseId(id);
//        result.setCollectionRecordList(records);
//        records.stream().peek(item -> {
//            if (StringUtils.isNotBlank(item.getCollectionUrlId())) {
//                List<Long> fileIds = Arrays.stream(item.getCollectionUrlId().split(","))
//                        .map(Long::parseLong)
//                        .collect(Collectors.toList());
//                item.setFileManager(fileManagerMapper.selectBatchIds(fileIds));
//            }
//        }).collect(Collectors.toList());
//        return result;
//    }

    //    @Override
//    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
//    public Boolean updateCaseInfo(CaseInformationEnterVO params) {
//
//        LoginUser loginUser = SecurityUtils.getLoginUser();
//        Long userId = loginUser.getUserId();
//        String username = loginUser.getUsername();
//        String nickName = loginUser.getUser().getNickName();
//        try {
//            //案件信息表----caseInformation
//            CaseInformation caseInformation = params.getCaseInformation();
//            Long caseId = caseInformation.getId();
//
//            //案件关联【委托方】----entrustingParty--从数据库获取出原来的委托方信息与通过接口传入的进行比对
//            List<ImpulseInformation> oldImpulse = informationMapper.selectByCaseId(params.getCaseInformation().getId(), 0);
//            List<ImpulseInformation> newImpulse = params.getEntrustingParty();
//            editImpulseInformation(oldImpulse, newImpulse, caseId);
//
//            //案件关联【相对方】----opposite
//            List<ImpulseInformation> oldOpposite = informationMapper.selectByCaseId(params.getCaseInformation().getId(), 1);
//            List<ImpulseInformation> newOpposite = params.getOpposite();
//            editImpulseInformation(oldOpposite, newOpposite, caseId);
//            caseInformation.setImpulseInformationIds(informationMapper.selectByCaseIdAdnDelFlag(params.getCaseInformation().getId()).stream().map(i -> i.getId().toString()).collect(Collectors.joining(",")));
//
//
//            //代理律师----attorneyAgent----修改原来的律师信息
//            CaseLawyer newAttorneyAgent = params.getAttorneyAgent();
//            if (BeanUtil.isNotEmpty(newAttorneyAgent)) {
//                if (newAttorneyAgent.getId() == null) {
//                    CaseLawyer oldCaseLawyer = caseLawyerMapper.selectAttorneyAgentLawByCaseId(params.getCaseInformation().getId());
//                    if (oldCaseLawyer != null) {
//                        oldCaseLawyer.setIsPrime(1);
//                        caseLawyerMapper.updateById(oldCaseLawyer);
//                    }
//                    newAttorneyAgent.setCaseId(params.getCaseInformation().getId());
//                    caseLawyerMapper.insert(newAttorneyAgent);
//                }
//            }
//
//
//            //协办律师----assistingLawyer
//            List<CaseLawyer> newAssistingLawyer = params.getAssistingLawyer(); //协办律师
//            List<CaseLawyer> oldAssistingLawyer = caseLawyerMapper.selectAssistingLawyerByCaseId(params.getCaseInformation().getId());
//            editAssistingLawyer(oldAssistingLawyer, newAssistingLawyer, caseId);
//            caseInformation.setCoLawyerIds(caseLawyerMapper.selectAllPrimeLawyerByCaseId(params.getCaseInformation().getId()).stream().map(c -> c.getLawyerId().toString()).collect(Collectors.joining(",")));
//
//            /**
//             * 收款记录----collectionRecordList
//             * 收款记录基本信息----collectionRecord
//             * 收款记录文件----fileManager
//             */
//            List<CollectionRecordEnterVO> newData = params.getCollectionRecordList();  //收款记录信息
//            List<CollectionRecordEnterVO> oldData = collectionRecordMapper.selectByCaseId(caseId);
//            List<Long> newIds = newData.stream().map(item -> item.getId()).collect(Collectors.toList());
//            List<Long> oldIds = oldData.stream().map(item -> item.getId()).collect(Collectors.toList());
//            //获取新添加的收款记录，计算出新的收款记录金额，在总金额中加上
//            List<CollectionRecordEnterVO> insert = newData.stream().filter(item -> item.getId() == null).peek(item -> item.setCaseId(caseId)).collect(Collectors.toList());
//            insert.forEach(item -> {
//                List<FileManager> fileManager = item.getFileManager().stream().peek(i -> i.setCaseId(caseId)).collect(Collectors.toList());
//                fileManagerService.saveBatch(fileManager);
//                item.setCollectionUrlId(fileManager.stream().map(i -> i.getId().toString()).collect(Collectors.joining(",")));
//                item.setUserId(userId);
//                item.setUserName(nickName);
//                String collect = loginUser.getUser().getRoles().stream().map(SysRole::getRoleKey).collect(Collectors.joining(","));
//                if (!collect.contains("FINANCE")) {
//                    item.setAuditStatus(0);
//                } else {
//                    item.setAuditStatus(1);
//                }
//
//            });
//            List<CollectionRecord> list = BeanUtil.copyToList(insert, CollectionRecord.class);
//            collectionRecordService.saveBatch(list);
//
//            //获取删除的收款记录,计算出删除的收款记录的金额，在总金额中减去
//            List<Long> deleteIds = (List<Long>) CollUtil.subtract(oldIds, newIds);
//            List<CollectionRecordEnterVO> deleteData = oldData.stream().filter(item -> CollUtil.contains(deleteIds, item.getId())).peek(item -> {
//                item.setDelFlag(1);
//                item.setStatus(1);
//            }).collect(Collectors.toList());
//
//
//            for (CollectionRecordEnterVO item : deleteData) {
//                //根据caseId和收款记录ID删除收款记录
//                collectionRecordMapper.removeByCaseIdAdnRecordId(item.getId(), caseId);
//                //根据caseId和收款记录ID删除收款记录文件
//                List<Long> fileIds = Arrays.stream(item.getCollectionUrlId().split(","))
//                        .map(Long::parseLong)
//                        .collect(Collectors.toList());
//                fileManagerMapper.removeByCaseIdAndFileIds(caseId, fileIds);
//            }
//            caseInformationMapper.updateById(caseInformation);
//        } catch (Exception e) {
//            throw new RuntimeException("保存失败");
//        }
//        return true;
//    }

    @Override
    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    public Boolean deleteCase(String ids) {
        try {
            //逻辑删除案件信息
            List<Long> longList = Arrays.stream(ids.split(","))
                    .map(Long::parseLong)
                    .collect(Collectors.toList());
            caseInformationMapper.removeByIds(longList);
            //逻辑删除案件相关律师信息
//            caseLawyerMapper.deleteLawyerByCaseId(longList);
            //逻辑删除收款记录
            collectionRecordMapper.removeByCaseId(longList);
            //逻辑删除案件相关文件
//            fileManagerMapper.removeByCaseId(longList);
            //将案件相关利冲信息设置为不需要检索，并将状态设置为停用、删除标志设置为已删除
            informationMapper.removeByCaseId(longList);
        } catch (Exception e) {
            throw new RuntimeException("案件删除失败");
        }
        return true;   //mybatis-plus自带的批量删除方法【逻辑删除】
    }

    @Override
    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    public Boolean caseArchived(CaseArchivedVO caseArchivedVO) {
        try {
            Long caseId = caseArchivedVO.getCaseId();
            caseInformationMapper.caseArchived(caseId);
            List<FileManager> fileManager = caseArchivedVO.getFileManager();
            fileManager.stream().peek(item -> {
                item.setCaseId(caseId);
                item.setFileType(0);
            }).collect(Collectors.toList());
            fileManagerService.saveBatch(fileManager);
        } catch (Exception e) {
            throw new RuntimeException("案件归档失败");
        }
        return true;
    }

    @Override
    public Boolean caseReview(Map<String, Object> params) {
        return caseInformationMapper.caseReview(params);
    }

    @Override
    public Boolean claimWithdrawal(Map<String, Object> params) {
        return caseInformationMapper.claimWithdrawal(params);
    }

    @Override
    public Boolean closeCase(Map<String, Object> params) {
        return caseInformationMapper.claimWithdrawal(params);
    }

    @Override
    public Integer caseClosed(CaseCloseSearcher searcher) {
        Integer result = caseInformationMapper.caseClosed(searcher);
        //案件结案后，需要将利冲信息表中的用户信息置为无需检索
        impulseInformationMapper.updateByCaseId(searcher.getId());
        return result;
    }

    @Override
    public Boolean caseInfoUpload(List<FileManager> fileManager) {
        fileManager.forEach(item -> item.setFileType(2));
        return fileManagerService.saveBatch(fileManager);
    }

    @Override
    public Map<String, List<CaseStatistics>> statisticsCaseStatus(Integer type) {
        return getStatisticsCaseStatusDate(type);
    }


    public Map<String, List<CaseStatistics>> getStatisticsCaseStatusDate(Integer type) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色【律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        String userId = "";
        if (CaseEnum.LAWYER.getRoleKey().equals(roleKey)) {
            userId = loginUser.getUserId().toString();
        }
        Map<String, List<CaseStatistics>> resMap = new HashMap<>();
        //案件状态统计
        List<CaseStatistics> caseStatus = new ArrayList<>();
        caseStatus.add(new CaseStatistics("利冲收案", caseInformationMapper.statisticsCaseStatus(userId, 0, type)));
        caseStatus.add(new CaseStatistics("正常收案", caseInformationMapper.statisticsCaseStatus(userId, 1, type)));
        caseStatus.add(new CaseStatistics("终止收案", caseInformationMapper.statisticsCaseStatus(userId, 2, type)));
        caseStatus.add(new CaseStatistics("案件撤销", caseInformationMapper.statisticsCaseStatus(userId, 3, type)));
        resMap.put("caseStatus", caseStatus);

        //收款状态统计
        List<CaseStatistics> collectionStatus = new ArrayList<>();
        collectionStatus.add(new CaseStatistics("未付款", caseInformationMapper.statisticsCaseCollectionStatus(userId, 0, type)));
        collectionStatus.add(new CaseStatistics("部分付款", caseInformationMapper.statisticsCaseCollectionStatus(userId, 1, type)));
        collectionStatus.add(new CaseStatistics("已结清", caseInformationMapper.statisticsCaseCollectionStatus(userId, 2, type)));
        collectionStatus.add(new CaseStatistics("逾期部分未付款", caseInformationMapper.statisticsCaseCollectionStatus(userId, 3, type)));
        resMap.put("collectionStatus", collectionStatus);

        //利冲状态
        List<CaseStatistics> impulseStatus = new ArrayList<>();
        impulseStatus.add(new CaseStatistics("未检索", caseInformationMapper.statisticsCaseImpulseStatus(userId, 0, type)));
        impulseStatus.add(new CaseStatistics("疑似利冲", caseInformationMapper.statisticsCaseImpulseStatus(userId, 1, type)));
        impulseStatus.add(new CaseStatistics("检索通过", caseInformationMapper.statisticsCaseImpulseStatus(userId, 2, type)));
        resMap.put("impulseStatus", impulseStatus);

        //审核状态
        List<CaseStatistics> auditStatus = new ArrayList<>();
        auditStatus.add(new CaseStatistics("未审核", caseInformationMapper.statisticsCaseAuditStatus(userId, 0, type)));
        auditStatus.add(new CaseStatistics("已审核", caseInformationMapper.statisticsCaseAuditStatus(userId, 1, type)));
        auditStatus.add(new CaseStatistics("审核不通过", caseInformationMapper.statisticsCaseAuditStatus(userId, 2, type)));
        resMap.put("auditStatus", auditStatus);
        //归档状态
        List<CaseStatistics> archiveStatus = new ArrayList<>();
        archiveStatus.add(new CaseStatistics("未归档", caseInformationMapper.statisticsCaseArchiveStatus(userId, 0, type)));
        archiveStatus.add(new CaseStatistics("已归档", caseInformationMapper.statisticsCaseArchiveStatus(userId, 1, type)));
        resMap.put("archiveStatus", archiveStatus);
        //结案状态
        List<CaseStatistics> finishStatus = new ArrayList<>();
        finishStatus.add(new CaseStatistics("未结案", caseInformationMapper.statisticsCaseFinishStatus(userId, 0, type)));
        finishStatus.add(new CaseStatistics("已结案", caseInformationMapper.statisticsCaseFinishStatus(userId, 1, type)));
        resMap.put("isFinishStatus", finishStatus);
        return resMap;
    }

    /**
     * 编辑协办律师
     *
     * @param oldData 旧的数据
     * @param newData 新的数据
     */
    public void editAssistingLawyer(List<CaseLawyer> oldData, List<CaseLawyer> newData, Long caseId) {
        List<CaseLawyer> insert = newData.stream().filter(c -> c.getId() == null).peek(c -> c.setCaseId(caseId)).collect(Collectors.toList());
        insert.stream().peek(new Consumer<CaseLawyer>() {
            @Override
            public void accept(CaseLawyer item) {
                item.setLawyerType(1);
            }
        }).collect(Collectors.toList());
        List<Long> oldIds = oldData.stream().map(c -> c.getId()).collect(Collectors.toList());
        List<Long> newIds = newData.stream().map(c -> c.getId()).collect(Collectors.toList());
        List<Long> deleteIds = (List<Long>) CollUtil.subtract(oldIds, newIds); //将原来的数据与现在数据进行比较，得到需要删除的ID
        if (CollUtil.isNotEmpty(insert)) {
            caseLawyerService.saveBatch(insert);
        }
//        if (CollUtil.isNotEmpty(deleteIds)) {
//            caseLawyerMapper.updateByIds(deleteIds);
//        }
    }

    /**
     * 编辑利冲信息
     *
     * @param oldData 旧的数据
     * @param newData 新的数据
     */
    public void editImpulseInformation(List<ImpulseInformation> oldData, List<ImpulseInformation> newData, Long caseId) {
        List<ImpulseInformation> insert = newData.stream().filter(item -> item.getId() == null).peek(item -> item.setCaseId(caseId)).collect(Collectors.toList());
        List<Long> oldIds = oldData.stream().map(item -> item.getId()).collect(Collectors.toList());
        List<Long> newIds = newData.stream().map(item -> item.getId()).collect(Collectors.toList());
        Collection<Long> updateIds = CollUtil.intersection(oldIds, newIds); //交集【UPDATE】
        Collection<Long> deleteIds = CollUtil.subtract(oldIds, newIds); //差集【DELETE】

        List<ImpulseInformation> updateDate = newData.stream().filter(item -> CollUtil.contains(updateIds, item.getId())).collect(Collectors.toList());
        List<ImpulseInformation> deleteData = oldData.stream().filter(item -> CollUtil.contains(deleteIds, item.getId())).collect(Collectors.toList());


        if (CollUtil.isNotEmpty(insert)) {
            impulseInformationService.saveBatch(insert);
        }
        if (CollUtil.isNotEmpty(updateDate)) {
            impulseInformationService.updateBatchById(updateDate);
        }
        if (CollUtil.isNotEmpty(deleteData)) {
            informationMapper.deleteBatchLogic(deleteData);
        }
    }


    /**
     * 案件信息录入接口
     *
     * @param searcher
     * @return
     */
    @Override
    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    public Integer enterCaseInfo2(CaseInformationEnterSearcher searcher) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        Long userId = loginUser.getUserId();
        String nickName = loginUser.getUser().getNickName();
        CaseInformation caseInformation = searcher.getCaseInformation();
        //生成案件编号
        caseInformation.setCaseNo(redisGenerateCaseNo.generateCaseNo(Integer.parseInt(caseInformation.getBusinessClassify())));
        Date paymentDeadline = caseInformation.getPaymentDeadline();
        if (Objects.nonNull(paymentDeadline)) {
            if (DateUtil.compare(paymentDeadline, new Date()) < 0 && caseInformation.getAmountReceivedSum().compareTo(caseInformation.getAmountReceivable()) < 0) {
                caseInformation.setCollectionStatus(3);
            }
        }
        Integer result = caseInformationMapper.insert(caseInformation);
        //获取案件id
        Long caseId = caseInformation.getId();
        // 录入委托方与相对方信息
        List<ImpulseInformation> entrustingParty = searcher.getEntrustingParty();
        List<ImpulseInformation> opposite = searcher.getOpposite();

        List<ImpulseInformation> impulseInformationList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(entrustingParty)) {
            impulseInformationList.addAll(entrustingParty.stream().peek(item -> {
                item.setCaseId(caseId);
                item.setIsClient(0);
            }).collect(Collectors.toList()));
        }
        if (!CollectionUtils.isEmpty(opposite)) {
            impulseInformationList.addAll(opposite.stream().peek(item -> {
                item.setCaseId(caseId);
                item.setIsClient(1);
            }).collect(Collectors.toList()));
        }
        impulseInformationService.saveBatch(impulseInformationList);
        List<Long> impulseInformationIds = impulseInformationList.stream().map(item -> item.getId()).collect(Collectors.toList());
        caseInformation.setImpulseInformationIds(StringUtils.join(impulseInformationIds, ","));
        caseInformationService.updateById(caseInformation);

        //录入收款记录
        List<CollectionRecordSearcher> collectionRecordList = searcher.getCollectionRecordList();
        List<CollectionRecord> collectionRecordListNew = new ArrayList<>();
        if (!CollectionUtils.isEmpty(collectionRecordList)) {
            for (CollectionRecordSearcher item : collectionRecordList) {
                item.setCaseId(caseId);
                item.setUserId(userId);
                item.setUserName(nickName);
                CollectionRecord collectionRecord = new CollectionRecord();
                BeanUtils.copyProperties(item, collectionRecord);
                collectionRecord.setAuditStatus(0);// 添加审核状态
                collectionRecordListNew.add(collectionRecord);
            }
        }
        collectionRecordService.saveBatch(collectionRecordListNew);
        return result;
    }

    @Override
    public CaseInformationEnterOverrideVO selectCaseInfoByIdOverride(Long id) {
        CaseInformationEnterOverrideVO resultVO = new CaseInformationEnterOverrideVO();
        CaseInformation caseInformation = caseInformationMapper.selectById(id);
        //处理委托律师及协办律师姓名
        List<SysUser> sysUserList = sysUserMapper.queryUserList();
        Map<Long, String> userIdAndNameMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(sysUserList)) {
            for (SysUser sysUser : sysUserList) {
                userIdAndNameMap.put(sysUser.getUserId(), sysUser.getNickName());
            }
        }
        caseInformation.setEntrustedLawyerName(userIdAndNameMap.get(caseInformation.getEntrustedLawyerId()));
        // 考虑协办律师可能为空的情况，需要进行判空处理
        if (!StringUtils.isEmpty(caseInformation.getCoLawyerIds())) {
            List<String> coLawyerIds = Arrays.asList(caseInformation.getCoLawyerIds().split(","));
            List<String> coLawyerNames = new ArrayList<>();
            for (String coLawyerId : coLawyerIds) {
                coLawyerNames.add(userIdAndNameMap.get(Long.parseLong(coLawyerId)));
            }
            caseInformation.setCoLawyerNames(StringUtils.join(coLawyerNames, ","));
        }
        resultVO.setUncollectedMoney(caseInformation.getAmountReceivable().subtract(caseInformation.getAmountReceivedSum()));
        resultVO.setCaseInformation(caseInformation);
        //案件涉及的委托方与相对方：
        //案件关联【委托方】----entrustingParty
        resultVO.setEntrustingParty(informationMapper.selectByCaseId(id, 0));

        //案件关联【相对方】----opposite
        resultVO.setOpposite(informationMapper.selectByCaseId(id, 1));

        //收款记录：
        List<CollectionRecordEnterVO> collectionRecordList = collectionRecordMapper.selectByCaseId(id);
        resultVO.setCollectionRecordList(collectionRecordList);
        return resultVO;
    }

    @Override
    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    public Integer updateCaseInfoOverride(CaseInformationEnterSearcher params) {
        /*
         * 修改接口可能存在修改的部分：
         * 1、案件基本信息
         * 2、案件关联的委托人、相对方，存在新增、修改、删除等情况
         * 3、案件关联的收款记录，存在新增、修改、删除等情况，案件信息表中的已收金额需要重新进行汇总
         * */
        LoginUser loginUser = SecurityUtils.getLoginUser();
        Long userId = loginUser.getUserId();
        String nickName = loginUser.getUser().getNickName();

        // 获取当前的案件信息
        CaseInformation caseInformation = params.getCaseInformation();
        Long caseId = caseInformation.getId();
        //查询所有的案件关联的委托方与相对方信息，在修改信息的时候不再去做利冲检索，只负责修改
        List<ImpulseInformation> currentImpulseInformationList = impulseInformationMapper.selectList(new QueryWrapper<ImpulseInformation>().lambda()
                .eq(ImpulseInformation::getCaseId, caseId)
                .eq(ImpulseInformation::getDelFlag, 0)
                .eq(ImpulseInformation::getCheckFlag, 1)
        );
        //前端传进来的相对方与委托方
        List<ImpulseInformation> entrustingParty = params.getEntrustingParty();
        List<ImpulseInformation> opposite = params.getOpposite();
        List<ImpulseInformation> impulseInformationList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(entrustingParty)) {
            impulseInformationList.addAll(entrustingParty);
        }
        if (!CollectionUtils.isEmpty(opposite)) {
            impulseInformationList.addAll(opposite);
        }
        //统一处理，对当前数据判断是需要新增、修改、还是删除
        editImpulseInformation(currentImpulseInformationList, impulseInformationList, caseId);
        //获取信息的利冲信息ids
        String impulseInformationIds = informationMapper.selectByCaseIdAdnDelFlag(caseId).stream().map(i -> i.getId().toString()).collect(Collectors.joining(","));

        //处理收款记录，没有收款记录id的是新增的，有id的是修改的，前端传的收款记录若不在收款记录列表中，则是删除的
        List<CollectionRecord> currentCollectionRecordList = collectionRecordMapper.selectList(new QueryWrapper<CollectionRecord>().lambda()
                .eq(CollectionRecord::getCaseId, caseId)
                .eq(CollectionRecord::getDelFlag, 0)
                .eq(CollectionRecord::getStatus, 0)
        );
        List<Long> currentCollectionRecordIdList = currentCollectionRecordList.stream().map(CollectionRecord::getId).collect(Collectors.toList());

        List<CollectionRecordSearcher> collectionRecordList = params.getCollectionRecordList();
        //重新计算已收金额
        BigDecimal amountReceivedSum = new BigDecimal("0");
        List<CollectionRecord> addCollectionRecordList = new ArrayList<>();
        List<CollectionRecord> updateCollectionRecordList = new ArrayList<>();
        List<CollectionRecord> deleteCollectionRecordList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(collectionRecordList)) {
            List<CollectionRecord> collectionRecordListNew = BeanUtil.copyToList(collectionRecordList, CollectionRecord.class);
            for (CollectionRecord item : collectionRecordListNew) {
                if (Objects.isNull(item.getId())) {
                    item.setCaseId(caseId);
                    item.setUserId(userId);
                    item.setUserName(nickName);
                    String roleKey = sysUserMapper.selectUserRoleByUserId(userId);
                    if ("CAIWU".equals(roleKey)) {
                        item.setAuditStatus(1);
                    } else {
                        item.setAuditStatus(0);
                    }
                    addCollectionRecordList.add(item);
                } else if (currentCollectionRecordIdList.contains(item.getId())) {
                    updateCollectionRecordList.add(item);
                } else {
                    deleteCollectionRecordList.add(item);
                }
                //重新计算已收金额
                if (Objects.nonNull(item.getAmountReceived())) {
                    amountReceivedSum = amountReceivedSum.add(item.getAmountReceived());
                }
            }
            collectionRecordService.saveBatch(addCollectionRecordList);
            collectionRecordService.updateBatchById(updateCollectionRecordList);
            collectionRecordService.removeBatchByIds(deleteCollectionRecordList);
        } else {
            //删除所有搜款记录
            amountReceivedSum = new BigDecimal("0");
            collectionRecordService.removeBatchByIds(currentCollectionRecordIdList);
            //更新案件信息中的已收金额
            caseInformationMapper.updateCaseAmount(caseId, amountReceivedSum);
        }


        //修改案件信息
        caseInformation.setAmountReceivedSum(amountReceivedSum);
        caseInformation.setImpulseInformationIds(impulseInformationIds);
        Integer result = caseInformationMapper.updateById(caseInformation);
        return result;
    }

}
