package com.tcctlo.law.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctlo.common.core.domain.model.LoginUser;
import com.tcctlo.common.utils.SecurityUtils;
import com.tcctlo.law.entity.CaseInformation;
import com.tcctlo.law.entity.DemandPaymentRecord;
import com.tcctlo.law.mapper.CaseInformationMapper;
import com.tcctlo.law.mapper.DemandPaymentRecordMapper;
import com.tcctlo.law.service.IDemandPaymentRecordService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
* <p>
* 金额催收记录 服务实现类
* </p>
*
* @author 
* @since 2025-02-27
*/
@Service
public class DemandPaymentRecordServiceImpl extends ServiceImpl<DemandPaymentRecordMapper,DemandPaymentRecord> implements IDemandPaymentRecordService {


    @Resource
    private DemandPaymentRecordMapper demandPaymentRecordMapper;

    @Resource
    private CaseInformationMapper caseInformationMapper;

    @Override
    public Page<DemandPaymentRecord> list(Integer pageNo, Integer pageSize) {
        return demandPaymentRecordMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public DemandPaymentRecord getById(Long id) {
        DemandPaymentRecord demandPaymentRecord = demandPaymentRecordMapper.selectById(id);
        return demandPaymentRecord;
    }

    @Override
    public Integer create(DemandPaymentRecord demandPaymentRecord) {
        return demandPaymentRecordMapper.insert(demandPaymentRecord);
    }

    @Override
    public Integer delete(Long id) {
        return demandPaymentRecordMapper.deleteById(id);
    }

    @Override
    public Integer update(DemandPaymentRecord demandPaymentRecord) {
        return demandPaymentRecordMapper.updateById(demandPaymentRecord);
    }

    @Override
    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    public Boolean caseCollection(Map<String, Long> params) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        Long userId = loginUser.getUserId();
        String username = loginUser.getUsername();
        String nickName = loginUser.getUser().getNickName();
        try {
            Long caseId = params.get("caseId"); //  案件ID
            //查询案件信息
            CaseInformation caseInformation = caseInformationMapper.selectById(caseId);
            String caseName = caseInformation.getCaseName();    //案件名称
            String caseNo = caseInformation.getCaseNo();    //案件编号
            BigDecimal amountReceivable = caseInformation.getAmountReceivable();    //案件应收金额
            BigDecimal amountReceivedSum = caseInformation.getAmountReceivedSum();  //案件已收金额
            Date paymentDeadline = caseInformation.getPaymentDeadline();    //付款期限 DateUtil.format(paymentDeadline,"yyyy-MM-dd")
            BigDecimal reserveMoney = caseInformation.getReserveMoney();    //预留款
            Date termOfConsultancyStart = caseInformation.getTermOfConsultancyStart();    //顾问期限开始时间
            Date termOfConsultancyEnd = caseInformation.getTermOfConsultancyEnd();    //顾问期限结束时间

//            String msg = caseName + "[" + caseNo + "]，应收金额为：" + amountReceivable + "，已收金额为：" + amountReceivedSum + "，付款期限：" + (paymentDeadline == null ? "无" : DateUtil.format(paymentDeadline, "yyyy-MM-dd")) + "，预留款：" + reserveMoney + "，顾问期限：" + (termOfConsultancy == null ? "无" : DateUtil.format(termOfConsultancy, "yyyy-MM-dd")) + "，剩余：" + amountReceivable.subtract(amountReceivedSum) + "未缴费，请尽快缴费";
            String msg = caseName + "[" + caseNo + "]，应收金额为：" + amountReceivable + "，已收金额为：" + amountReceivedSum + "，付款期限：" + (paymentDeadline == null ? "无" : DateUtil.format(paymentDeadline, "yyyy-MM-dd")) + "，预留款：" + reserveMoney + "，顾问期限开始时间：" + (termOfConsultancyStart == null ? "无" : DateUtil.format(termOfConsultancyStart, "yyyy-MM-dd")) + "顾问期限结束时间：" + (termOfConsultancyEnd == null ? "无" : DateUtil.format(termOfConsultancyEnd, "yyyy-MM-dd")) + "，剩余：" + amountReceivable.subtract(amountReceivedSum) + "未缴费，请尽快缴费";

            DemandPaymentRecord demandPaymentRecord = new DemandPaymentRecord();
            demandPaymentRecord.setCaseId(caseId);
            demandPaymentRecord.setCollectionContent(msg);
            demandPaymentRecord.setCollectionUserId(userId);
            demandPaymentRecord.setCollectionUserName(nickName);
            demandPaymentRecordMapper.insert(demandPaymentRecord);
            return true;
        }catch (Exception e){
            e.printStackTrace();
            throw new RuntimeException("催费失败");
        }
    }

    @Override
    public boolean setRead(Map<String, Object> params) {
        try {
            Long caseId = Long.parseLong(params.get("caseId").toString());
            String idsStr = (String) params.get("ids");
            List<Long> ids = Arrays.stream(idsStr.split(",")).map(Long::parseLong).collect(Collectors.toList());
            demandPaymentRecordMapper.setRead(caseId, ids);
            return true;
        }catch (Exception e){
            e.printStackTrace();
            throw new RuntimeException("设置已读失败");
        }
    }

}
