package com.tcctlo.law.service.impl;

import cn.hutool.core.date.DateUtil;
import com.tcctlo.common.core.domain.model.LoginUser;
import com.tcctlo.common.utils.SecurityUtils;
import com.tcctlo.law.CaseEnum;
import com.tcctlo.law.entity.CollectionRecord;
import com.tcctlo.law.entity.IndexStatistics;
import com.tcctlo.law.mapper.CaseInformationMapper;
import com.tcctlo.law.mapper.CollectionRecordMapper;
import com.tcctlo.law.searcher.TimeSearcher;
import com.tcctlo.law.service.ICollectionRecordService;
import com.tcctlo.law.service.IIndexService;
import com.tcctlo.law.vo.IndexLawCaseMoneyInfo;
import com.tcctlo.law.vo.IndexLawInfo;
import com.tcctlo.system.mapper.SysUserMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.BiConsumer;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class IIndexServiceImpl implements IIndexService {

    @Resource
    private CaseInformationMapper caseInformationMapper;

    @Resource
    private CollectionRecordMapper recordMapper;

    /**
     * 系统用户mapper
     */
    @Resource
    private SysUserMapper sysUserMapper;

    @Override
    public List<IndexStatistics> caseTypeStatistics(Date startTime, Date endTime) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色【律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        String userId = "";
        if (CaseEnum.LAWYER.getRoleKey().equals(roleKey)) {
            userId = loginUser.getUserId().toString();
        }
        List<IndexStatistics> res = new ArrayList<>();
        res.add(new IndexStatistics("民商事业务", Double.valueOf(caseInformationMapper.selectCountByBusinessType(userId, 0, startTime, endTime))));
        res.add(new IndexStatistics("刑事辩护业务", Double.valueOf(caseInformationMapper.selectCountByBusinessType(userId, 1, startTime, endTime))));
        res.add(new IndexStatistics("行政复议及诉讼业务", Double.valueOf(caseInformationMapper.selectCountByBusinessType(userId, 2, startTime, endTime))));
        res.add(new IndexStatistics("企业常年法律顾问", Double.valueOf(caseInformationMapper.selectCountByBusinessType(userId, 3, startTime, endTime))));
        res.add(new IndexStatistics("企业专项法律顾问", Double.valueOf(caseInformationMapper.selectCountByBusinessType(userId, 4, startTime, endTime))));
        res.add(new IndexStatistics("其他非诉讼业务", Double.valueOf(caseInformationMapper.selectCountByBusinessType(userId, 5, startTime, endTime))));
        res.add(new IndexStatistics("劳动争议仲裁", Double.valueOf(caseInformationMapper.selectCountByBusinessType(userId, 6, startTime, endTime))));
        res.add(new IndexStatistics("商事仲裁", Double.valueOf(caseInformationMapper.selectCountByBusinessType(userId, 7, startTime, endTime))));
        return res;
    }

    @Override
    public List<IndexStatistics> conflictStatistics(Date startTime, Date endTime) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色【律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        String userId = "";
        if (CaseEnum.LAWYER.getRoleKey().equals(roleKey)) {
            userId = loginUser.getUserId().toString();
        }

        List<IndexStatistics> res = new ArrayList<>();

        res.add(new IndexStatistics("利冲收案", Double.valueOf(caseInformationMapper.conflictStatistics(userId, 0, startTime, endTime))));
        res.add(new IndexStatistics("正常收案", Double.valueOf(caseInformationMapper.conflictStatistics(userId, 1, startTime, endTime))));
        res.add(new IndexStatistics("终止收案", Double.valueOf(caseInformationMapper.conflictStatistics(userId, 2, startTime, endTime))));
        res.add(new IndexStatistics("案件撤销", Double.valueOf(caseInformationMapper.conflictStatistics(userId, 3, startTime, endTime))));
        return res;
    }

    @Override
    public List<IndexStatistics> costStatistics(Date startTime, Date endTime) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色【律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        String userId = "";
        if (CaseEnum.LAWYER.getRoleKey().equals(roleKey)) {
            userId = loginUser.getUserId().toString();
        }

        List<IndexStatistics> res = new ArrayList<>();
        res.add(new IndexStatistics("未收款", Double.valueOf(caseInformationMapper.outstandingPayment(userId, startTime, endTime))));
        res.add(new IndexStatistics("部分收费", Double.valueOf(caseInformationMapper.partialCharge(userId, startTime, endTime))));
        res.add(new IndexStatistics("付款逾期", Double.valueOf(caseInformationMapper.overdue(userId, startTime, endTime))));
        return res;
    }

    @Override
    public List<IndexStatistics> earningsTrend(Date startTime, Date endTime) {

        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色【律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        String userId = "";
        if (CaseEnum.LAWYER.getRoleKey().equals(roleKey)) {
            userId = loginUser.getUserId().toString();
        }

        List<CollectionRecord> list = recordMapper.earningsTrend(userId, startTime, endTime);
        Map<Date, List<CollectionRecord>> map = list.stream().collect(Collectors.groupingBy(item -> DateUtil.parse(DateUtil.format(item.getCollectionDate(), "yyyy-MM-dd"))));
        TreeMap<Date, List<CollectionRecord>> treeMap = new TreeMap<>(map);
        List<IndexStatistics> res = new ArrayList<>();
        TreeMap<String, List<CollectionRecord>> itemMap = new TreeMap<>();
        treeMap.forEach((date, collectionRecords) -> itemMap.put(DateUtil.format(date, "yyyy-MM-dd"), collectionRecords));
        itemMap.forEach((s, collectionRecords) -> {
            IndexStatistics indexStatistics = new IndexStatistics();
            indexStatistics.setName(s);
            indexStatistics.setValue(collectionRecords.stream().mapToDouble(item -> item.getAmountReceived().doubleValue()).sum());
            res.add(indexStatistics);
        });
        return res;
    }

    @Override
    public List<IndexLawCaseMoneyInfo> lawInfo(TimeSearcher searcher) {
        List<IndexLawInfo> indexLawInfos = caseInformationMapper.selectLawInfo();   //所有的律师信息
        List<IndexLawCaseMoneyInfo> statcicList = caseInformationMapper.staticCaseNumAndMoney(searcher);
        Map<Long, IndexLawCaseMoneyInfo> indexLawCaseMoneyInfoMap = statcicList.stream().collect(Collectors.toMap(IndexLawCaseMoneyInfo::getLawyerId, Function.identity()));
        List<IndexLawCaseMoneyInfo> voList = new ArrayList<>();
        for (IndexLawInfo item : indexLawInfos) {
            IndexLawCaseMoneyInfo indexLawCaseMoneyInfo = new IndexLawCaseMoneyInfo();
            indexLawCaseMoneyInfo.setLawyerId(item.getUserId());
            indexLawCaseMoneyInfo.setLawName(item.getNickName());
            if (indexLawCaseMoneyInfoMap.containsKey(item.getUserId())) {
                indexLawCaseMoneyInfo.setCaseNum(indexLawCaseMoneyInfoMap.get(item.getUserId()).getCaseNum());
                indexLawCaseMoneyInfo.setAmountReceived(indexLawCaseMoneyInfoMap.get(item.getUserId()).getAmountReceived());
            }
            voList.add(indexLawCaseMoneyInfo);
        }
        return voList;
    }
}
